import java.util.*;
import java.lang.reflect.Array;
 
public class DWvStack extends M865Stack {
	protected Vector v;
	
	public DWvStack(int capacity, int increment) {
		v = new Vector(capacity, increment);
	}
	
	public DWvStack(int capacity) {
		v = new Vector(capacity);
	}
	
	public DWvStack() {
		v = new Vector();
	}
	
	protected DWvStack(Vector v) {
		this.v = (Vector)v.clone();
	}
	
	public void push(Object o) {
	// pushes an object onto the stack.
		v.add( o );
	}
	
	public Object pop() {
	// removes and returns the object on the top of the stack.
	// returns null if the stack is empty.
		if (v.isEmpty()) {
			return null;
		} else {
			Object x = v.lastElement();
			v.removeElementAt( v.size()-1 );
			return x;
		}
	}
	
	public Object peek() {
	// returns the object on the top of the stack.
	// returns null if the stack is empty.
		if (v.isEmpty()) {
			return null;
		} else {
			return (v.lastElement());
		}
	}

	public boolean add(Object o) {
	// Ensures that this collection contains the specified element.
		this.push( o );
		return true;
	}
	
	public boolean addAll(Collection c) {
	// Adds all of the elements in the specified collection to this stack
		Iterator ci = c.iterator();
		boolean changed = ci.hasNext();
		while (ci.hasNext()) {
			this.push( ci.next() );
		}
		return changed;
	}
	
	public void clear() {
	// Removes all of the elements from this collection (optional operation). 
		v.clear();
	}
		
	public Object clone() {
	// Returns a copy of this stack
		return (new DWvStack(v));
	}
	
	public boolean contains(Object o) {
	// Returns true if this collection contains the specified element.  
		return v.contains(o);
	}
	
	public boolean containsAll(Collection c) {
	// Returns true if this collection contains all of the elements in the
	// specified collection. 
		return v.containsAll(c);
	}
	 
	public boolean equals(Object o) {
	// Compares the specified object with this collection for equality. 
		boolean b = (o instanceof DWvStack) && (this.size() == ((DWvStack)o).size());
		if (b) {
			Iterator i = this.iterator();
			Iterator j = ((DWvStack)o).iterator();
			while (b && i.hasNext()) {
				b = (i.next()).equals(j.next());
			}
		}
		return b;
	}
	
	public int hashCode() {
	// Returns the hash code value for this collection.  
		return v.hashCode();
	}
	
	public boolean isEmpty() {
	// Returns true if this collection contains no elements.  
		return v.isEmpty();
	}
	
	public class DWvStackIterator implements Iterator {
		private Vector v;
		private int index;
		private int check;
		
		DWvStackIterator(Vector v) {
			this.v = v;
			index = v.size();
			check = v.hashCode();
		}

		public boolean hasNext() {
			return (0 < index);
		}
		
		public Object next() {
			return v.get(--index);
		}
		
		public void remove() {
	// Removes the last element returned by the iterator.
	// This is an optional operation which is not implemented for DWvStackIterator.
			throw( new UnsupportedOperationException("remove is not implemented for the general M865Stack"));
		}
	}

	public Iterator iterator() {
	// Returns an iterator over the elements in this collection.  
		return new DWvStackIterator(v);
	}
	
	public int size() {
	// Returns the number of elements in this collection.  
		return v.size();
	}
	
	public Object[] toArray() {
	// Returns an array containing all of the elements in this collection.
		Object[] A = new Object[v.size()];
		int i=v.size(); 
		int k=0;
		while (0<i) {
			A[k++] = v.get(--i);
		}
		return A;
	}
	
	public Object[] toArray(Object[] a) {
	// Returns an array containing all of the elements in this collection; 
	//the runtime type of the returned array is that of the specified array.
		int i = v.size();
		int k = 0;
		if (i <= a.length) {
			while (0<i) {
				a[k++] = v.get(--i);
			}
			if (v.size() < a.length) {
				a[k] = null;
			}
			return a;
		} else {
			Class c = a.getClass();
			c = c.getComponentType();
			Object[] A = (Object[])Array.newInstance(c, i);
			while (0<i) {
				A[k++] = v.get(--i);
			}
			return A;
		}
	}
	
	public String toString() {
		String s = "DWvStack {";
		Iterator i = this.iterator();
		while (i.hasNext()) {
			s += "\n	" + i.next().toString();
		}
		return (s + "\n}");
	}
	
	public static void main(String[] args) {
		DWvStack s1 = new DWvStack();
		String name;
		
		s1.push("Alice");
		s1.add("Bob");
		s1.push("Carol");
		System.out.println(s1);
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("Empty? " + s1.isEmpty());
		 
		System.out.println("\ns1 Iterator test:");
		Iterator iter = s1.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		
		System.out.println("\nLooking at three stacks");
		DWvStack s2 = new DWvStack();
		s2.addAll(s1);
		System.out.println("\ns2 Iterator test:");
		iter = s2.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		DWvStack s3 = new DWvStack();
		s3.addAll(s2);
		System.out.println("\ns3 Iterator test:");
		iter = s3.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}

		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1 equals s3? " + s1.equals(s3));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());
		System.out.println("s3.hashCode: " + s3.hashCode());
		System.out.println("Is Bob in s1? " + s1.contains("Bob"));
		s1.add("Doug");
		System.out.println(s1.peek());
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());
		System.out.println("s3.hashCode: " + s3.hashCode());
		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1 equals s3? " + s1.equals(s3));
		System.out.println("Is s1 contained in s2? " + s2.containsAll(s1));
		System.out.println("Is s2 contained in s1? " + s1.containsAll(s2));
		System.out.println("Is s2 contained in s3? " + s3.containsAll(s3));
		
		System.out.println("\nLooking at cloning");
		DWvStack s4 = (DWvStack)s1.clone();
		System.out.println("s1 equals s4? " + s1.equals(s4));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s4.hashCode: " + s4.hashCode());
		System.out.println("s4.pop: " + s4.pop());
		System.out.println("s1 equals s4? " + s1.equals(s4));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s4.hashCode: " + s4.hashCode());

		System.out.println("\nLooking at an array and a stack");
		Object[] names = s1.toArray();
		for (int i=0; i<names.length; i++) {
			System.out.println(names[i]);
		}
		String[] moreNames = (String[])s1.toArray(new String[5]);
		System.out.println("moreNames.length: " + moreNames.length);
		for (int i=0; i<moreNames.length; i++) {
			System.out.println(moreNames[i]);
		}


		System.out.println("\nEmptying the stacks");
		System.out.println(s1.pop());
		System.out.println(s1.hashCode());
		System.out.println(s1.pop());
		System.out.println(s1.pop());
		System.out.println(s1.pop());
		System.out.println("Empty? " + s1.isEmpty());
		s2.clear();
		System.out.println("Empty? " + s2.isEmpty());
		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());

	}
}