import java.util.*;

public class DWhvPriorityQueue extends M865PriorityQueue {
	protected Vector v;
	
	
	public DWhvPriorityQueue(int capacity, int increment) {
		v = new Vector(capacity, increment);
	}
	
	public DWhvPriorityQueue(int capacity) {
		v = new Vector(capacity);
	}
	
	public DWhvPriorityQueue() {
		v = new Vector();
	}
	
	protected DWhvPriorityQueue(Vector v, long h) {
		this.hash = h;
		this.v = (Vector)v.clone();
	}

	protected void bubbleUp(int child) {
		int parent;
		Comparable cPQI, pPQI;
		cPQI = (Comparable)v.elementAt(child);
		while (0 < child) {
			parent = (child-1)/2;
			pPQI = (Comparable)v.elementAt(parent);
			if (pPQI.compareTo(cPQI) <= 0) {
				// The parent has a higher priority than the child, so
				break;
			}
			v.setElementAt(pPQI, child);
			v.setElementAt(cPQI, parent);
			child = parent;
		}
	}
	
	public void enqueue(Comparable pqi) {
	// Add this Comparable to the priority queue.
		hash = (hash + pqi.hashCode()) % Integer.MAX_VALUE;
		v.add( pqi );
		bubbleUp(v.size()-1);
	}
	
	protected void siftDown(int parent) {
		int child, temp;
		Comparable pPQI, cPQI, tPQI;
		pPQI = (Comparable)v.elementAt(parent);
		while (2*parent+1 < v.size()) {
			// There is at least one child
			child = 2*parent + 1;
			cPQI = (Comparable)v.elementAt(child);
			if (child+1 < v.size()) {
				// There is a second child
				temp = child+1;
				tPQI = (Comparable)v.elementAt(temp);
				if (tPQI.compareTo(cPQI) <= 0) {
					// The second child has the higher priority,
					// so swap it with the first child
					child = temp;
					cPQI = tPQI;
				}
			}
			if (pPQI.compareTo(cPQI) <= 0) {
				// The parent has higher priority than the best child, so
				break;
			}
			v.setElementAt(cPQI, parent);
			v.setElementAt(pPQI, child);
			parent = child;
		}
	}
	
	public Comparable dequeue() {
	//  Remove and return a Comparable of highest priority.
	//  Return null if the priority queue is empty.
		if (v.size() <= 0) {
			return null;
		} else if (v.size() == 1) {
			Comparable obj = (Comparable)v.remove(0);
			hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
			return obj;
		} else {
			Comparable obj = (Comparable)v.get(0);
			hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
			Comparable pqi = (Comparable)v.remove(v.size()-1);
			v.setElementAt(pqi,0);
			siftDown(0);
			return obj;
		}
	}

	public Comparable peek() {
	//  Return a Comparable of highest priority.
		if (v.isEmpty()) {
			return null;
		} else {
			return ( (Comparable)v.get(0) );
		}
	}
/*
	public void adjustPriority(Comparable obj, Comparable priority) {
	//  Finds the object and adjusts its priority.
		Comparable pqi = null;
		int index = -1;
		for (int i=0; i<v.size(); i++) {
			pqi = (Comparable)v.elementAt(i);
			if (obj.equals(pqi)) {
				index = i;
				break;
			}
		}
		if (0 <= index) {
			boolean higher = pqi.adjustPriority(priority);
			v.setElementAt(pqi, index);
			if ( higher ) {
				// The priority has been raised, so
				bubbleUp(index);
			} else {
				// The priority has been lowered, so
				siftDown(index);
			}
		}
	}
*/
	
	public void clear() {
	// Removes all of the elements from this collection (optional operation). 
		hash = 0;
		v.clear();
	}
		
	public Object clone() {
	// Returns a copy of this queue
		return (new DWhvPriorityQueue(v, hash));
	}

	public boolean contains(Object obj) {
	// Returns true if this collection contains the specified Comparable.
	// This method overrides the corresponding method of the abstract class,
	// because that method uses a DWhvPriorityQueueIterator which is inefficient
	// for this heap-vector implementation.
		boolean b = false;
		for (int i=0; i<v.size(); i++) {
			b = obj.equals( v.get(i) );
			if ( b ) {
				break;
			}
		}
		return b;
	}


	public boolean equals(Object o) {
	// Compares the specified object with this collection for equality. 
		boolean b = (o instanceof M865PriorityQueue) && (this.size() == ((M865PriorityQueue)o).size());
		if (b) {
			Iterator i = this.iterator();
			Iterator j = ((M865PriorityQueue)o).iterator();
			while (b && i.hasNext()) {
				b = (i.next()).equals(j.next());
			}
		}
		return b;
	}
	
	public class DWhvPriorityQueueIterator implements Iterator {
		private DWhvPriorityQueue q, myPQ;
		private long check;
		private Comparable last;
		
		public DWhvPriorityQueueIterator(DWhvPriorityQueue q) {
			this.q = q;
			myPQ = (DWhvPriorityQueue)q.clone();
			check = q.hash;
			last = null;
		}

		public boolean hasNext() {
			//  Check that the queue has not been altered during the processing
			//  of this iterator
			if (check == q.hash) {
				return (0 < myPQ.size());
			} else {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
		}
		
		public Object next() {
			if (check == q.hash) {
				last = myPQ.dequeue();
				return last;
			} else {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
		}

		public void remove() {
			// Removes the last element returned by the iterator.
			if (check != q.hash) {
				throw( new ConcurrentModificationException("This DWhvPriorityQueue has been altered"));
			}
			if (last != null) {
				q.remove(last);
			}
		}
	}

	public Iterator iterator() {
	// Returns an iterator over the elements in this collection.  
		return new DWhvPriorityQueueIterator(this);
	}
	
	public boolean remove(Object obj) {
	// Remove the specified object from the priority queue.
		if (!(obj instanceof Comparable)) {
			throw( new ClassCastException("Can only remove Comparables"));
		}
		Comparable pqi = null;
		int index = -1;
		for (int i=0; i<v.size(); i++) {
			pqi = (Comparable)v.elementAt(i);
			if (obj.equals(pqi)) {
				// Found the object, so
				index = i;
				break;
			}
		}
		if (0 <= index) {
			Comparable old = pqi;
			pqi = (Comparable)v.remove(v.size()-1);
			if (index < v.size()) {
				v.setElementAt(pqi, index);
				if (pqi.compareTo(old) <= 0) {
					// The priority has been raised, so
					bubbleUp(index);
				} else {
					// The priority has been lowered, so
					siftDown(index);
				}
			}
			// The priority queue has been changed.
			return true;
		} else {
			// The priority queue is unchanged.
			return false;
		}
	}
	
	public int size() {
	//  Return the number of items in the queue.
		return v.size();
	}

	
	public String toString() {
		String s = super.toString();
		return( "DWhv" + s.substring(4, s.length()) );
	}

	protected String dumpVector() {
		String s = "DWhvPriorityQueue {";
		for (int i=0; i<v.size(); i++) {
			s += "\n " + v.get(i).toString();
		}
		return (s + "\n}");
	}
	
	public static void main(String args[]) {
		DWhvPriorityQueue pq = new DWhvPriorityQueue();
		
		pq.enqueue("0101");
		pq.enqueue("0011");
		pq.enqueue("00010111");
		System.out.println(pq);
		StringBuffer label = new StringBuffer("0");
		label.append((String)pq.dequeue());
		label.append((String)pq.dequeue());
		label.append((String)pq.dequeue());
		label.append("1");
		System.out.println("New label: " + label);
	}
}
