import java.io.*;
import java.util.*;

//*** Change YIGraph to (Your Initials)Graph ***//

public class YIGraph extends M865Graph {

	protected boolean updateComponentCount;	// flag for updating the component count
	protected int nC;									// the number of components
	protected boolean cyclic;						// does this graph have cycles
	protected boolean updateSpanningTree;		// flag for updating the spanning tree
	protected TreeSet spanningTreeEdges;		// the edges of a spanning tree
	protected boolean updateNeighborhoods;		// flag for updating the list of neighborhoods

	public YIGraph(M865Graph graph) {
		super(graph);
		updateComponentCount = true;
		updateSpanningTree = true;
		updateNeighborhoods = true;
		nC = 0;
		cyclic = false;
		spanningTreeEdges = null;
	}

	public YIGraph(SortedSet vertexSet, SortedSet edgeList) {
		super(vertexSet, edgeList);
		updateComponentCount = true;
		updateSpanningTree = true;
		updateNeighborhoods = true;
		nC = 0;
		cyclic = false;
		spanningTreeEdges = null;
	}

	public YIGraph(SortedSet vertexSet) {
		this(vertexSet, new TreeSet());
	}
	
	public YIGraph() {
		this(new TreeSet(), new TreeSet());
	}

	public boolean addVertex(M865Vertex v) {
		if (super.addVertex(v)) {
			updateComponentCount = true;
			updateSpanningTree = true;
			updateNeighborhoods = true;
			return true;
		} else {
			return false;
		}
	}

	protected void buildNeighborhoods() {
		neighborhoods = new TreeMap();
		Iterator iter = vertexSet.iterator();
		while (iter.hasNext()) {
			neighborhoods.put((M865Vertex)iter.next(), new TreeSet());
		}
		TreeSet neighbors;
		iter = edgeList.iterator();
		while (iter.hasNext()) {
			M865Edge e = (M865Edge)iter.next();
			M865Vertex s = (M865Vertex)e.getSource();
			M865Vertex t = (M865Vertex)e.getTarget();
			neighbors = (TreeSet)neighborhoods.get(s);
			neighbors.add(t);
			neighbors = (TreeSet)neighborhoods.get(t);
			neighbors.add(s);
		}
		updateNeighborhoods = false;
	}

	public SortedSet neighborhoodOf(M865Vertex vertex) {
		if (updateNeighborhoods) {
			buildNeighborhoods();
		}
		return (TreeSet)neighborhoods.get(vertex);
	}

	public boolean removeVertex(M865Vertex v) {
		if (super.removeVertex(v)) {
			updateComponentCount = true;
			updateSpanningTree = true;
			updateNeighborhoods = true;
			return true;
		} else {
			return false;
		}
	}

	public boolean addEdge(M865Edge e) {
		if (super.addEdge(e)) {
			updateComponentCount = true;
			updateSpanningTree = true;
			updateNeighborhoods = true;
			return true;
		} else {
			return false;
		}
	}

	public boolean removeEdge(M865Edge e) {
		if (super.removeEdge(e)) {
			updateComponentCount = true;
			updateSpanningTree = true;
			updateNeighborhoods = true;
			return true;
		} else {
			return false;
		}
	}

	protected int countComponents(boolean saveTree) {
		if (saveTree) {
			spanningTreeEdges = new TreeSet();
		}
		nC = vertexSet.size();
	
		/***
			Here is where you should insert your code for counting
			the components and for saving the edges of the spanning
			spanning tree (when saveTree is true)
		***/
	
		updateComponentCount = false;
		if (saveTree) {
			updateSpanningTree = false;
		} else {
			updateSpanningTree = true;
		}
		return nC;
	}		
					
	public Object clone() {
		return new YIGraph((SortedSet)((TreeSet)vertexSet).clone(), (SortedSet)((TreeSet)edgeList).clone());
	}

	public int getNumberOfComponents() {
		if (updateComponentCount) {
			return countComponents(false);
		} else {
			return nC;
		}
	}	

	public boolean hasCycles() {
		if (updateComponentCount) {
			countComponents(false);
		} 
		return cyclic;
	}

	public M865Graph getSpanningTree() {
		if (updateSpanningTree) {
			countComponents(true);
		}
		if (nC == 1) {
			return (new YIGraph(vertexSet, spanningTreeEdges));
		} else {
			return null;
		}
	}

	public int degreeOf(M865Vertex vertex) {
		if (updateNeighborhoods) {
			buildNeighborhoods();
		}
		/***
			Here you should insert your code to determine the
			degree from the size of the vertex's neighborhood.
		***/
		return 0;
	}

	public static void main (String args[])
		throws IOException {

		M865GraphReader graphReader;
		if (0 < args.length) {
			graphReader = new M865GraphReader(args[0]);
		} else {
			graphReader = new M865GraphReader();
		}
		graphReader.readEdgeList();
		graphReader.dump();
		
		M865Graph g = graphReader.getGraph();
		System.out.println(g);
		System.out.println("nC: " + g.getNumberOfComponents());
		System.out.println("Connected: " + g.isConnected());
		System.out.println("Has cycles: " + g.hasCycles());
		SortedSet v = g.getVertexSet();
		Iterator iter = v.iterator();
		while (iter.hasNext()) {
			M865Vertex vertex = (M865Vertex)iter.next();
			System.out.println("Degree of "+vertex+" is "+g.degreeOf(vertex));
		}
		g.printNeighborhoods();

		M865Graph tree = g.getSpanningTree();
		System.out.println("Spanning tree: ");
		System.out.println(tree);		
	}
}