import java.io.*;
import java.util.*;


public class YITree extends M865Tree {

	public YITree(M865Tree tree) {
		super(tree);
	}
	
	public YITree(M865Graph graph) {
		super(graph);
	}

	public YITree(SortedSet vertexSet, SortedSet edgeList) {
		super(vertexSet, edgeList);
	}
		
	public YITree() {
		super();
	}

	public Object clone() {
		return new YITree((SortedSet)((TreeSet)vertexSet).clone(), (SortedSet)((TreeSet)edgeList).clone());
	}

	protected void buildCertificate(YITree t) {
		// Make a temporary copy of the tree which we can manipulate.
		YITree tree = (YITree)t.clone();

		// Initialize the labels for each vertex
		TreeMap labelOf = new TreeMap();
		Iterator iter = tree.getVertexSet().iterator();
		while (iter.hasNext()) {

			/***
				Your Initialization statement(s) go here
			***/

		}

		// Make multiple passes over the tree, stripping leaves and building the labels
		M865PriorityQueue pq = new DWhvPriorityQueue(tree.getVertexSet().size());
		Vector leaves = new Vector();
		// The next two lines are for debugging purposes
		int pass = 0;
		String s, pad = "             ";
		while (2 < tree.getVertexSet().size()) {
			System.out.println("\nPass " + pass++);

			// Build a new label for every interior vertex 
			iter = tree.getVertexSet().iterator();
			while (iter.hasNext()) {
				M865Vertex v = (M865Vertex)iter.next();
				// The next two lines are for debugging purposes
				s = v.toString() + ":";
				System.out.println(s + pad.substring(s.length()) + (String)labelOf.get(v));

				/***
					For every vertex that is not a leaf take the sublabel
					obtained by stripping off the first and last letters, and
					concatenate in lexicographic order the sublabel and the
					labels of all leaves that are adjacent to this vertex.
					The new label for this vertex is the concatenated label with
					"0" prepended and "1" appended.
				***/

			}

			// Remove all the leaves that were found on this pass
			iter = leaves.iterator();
			while (iter.hasNext()) {
				tree.removeVertex((M865Vertex)iter.next());
			}
			leaves.clear();
		}
		center = (M865Vertex)tree.getVertexSet().first();
		String label1 = (String)labelOf.get(center);
		String label2;
		if (tree.getVertexSet().size() == 2) {
			label2 = (String)labelOf.get(tree.getVertexSet().last());
			if (0 < label1.compareTo(label2)) {
				certificate = label2 + label1;
			} else {
				certificate = label1 + label2;
			}
		} else {
			certificate = label1;
		}
		updateCertificate = false;
	}	


	protected void buildRootedSubTree(M865Vertex p) {
		M865TreeNode pNode = (M865TreeNode)nodeOf.get(p);
		M865Vertex g = pNode.getParent().getVertex();
		SortedSet neighbors = neighborhoodOf(p);

		/***
			Add the code to recursively build the subtree for this node.
			Note that as you walk through the neighbors you must exclude
			that node which happens to be the parent of this node.
		***/

	}
	
	public M865TreeNode buildRootedTree(M865Vertex rootVertex) {
		if (!vertexSet.contains(rootVertex)) {
			throw (new UnsupportedOperationException("Vertex is not in this tree"));
		}
		// Initialize the nodeOf TreeMap.  
		// This provides the M865TreeNode corresponding to each vertex.
		nodeOf = new TreeMap();
		Iterator iter = vertexSet.iterator();
		while (iter.hasNext()){
			M865Vertex v = (M865Vertex)iter.next();
			nodeOf.put(v, new M865TreeNode(v, null, null, null));
		}
		if (updateNeighborhoods) {
			buildNeighborhoods();
		}
		root = (M865TreeNode)nodeOf.get(rootVertex);
		SortedSet neighbors = neighborhoodOf(rootVertex);
		iter = neighbors.iterator();
		while (iter.hasNext()) {
			M865Vertex v = (M865Vertex)iter.next();
			root.addChild((M865TreeNode)nodeOf.get(v));
			buildRootedSubTree(v);
		}
		updateRootedTree = false;
		return root;
	}

	public String getCertificate() {
		if (updateCertificate) {
			buildCertificate(this);
		}
		return certificate;
	}

	public M865Vertex getCenter() {
		if (updateCertificate) {
			buildCertificate(this);
		}
		return center;
	}

	public void printTree() {
		if (updateRootedTree) {
			M865Vertex rootVertex = (M865Vertex)vertexSet.first();
			buildRootedTree(rootVertex);
		}
		root.printTree();
	}
	
	public static void main (String args[])
		throws IOException {

		M865GraphReader graphReader;
		if (0 < args.length) {
			graphReader = new M865GraphReader(args[0]);
		} else {
			graphReader = new M865GraphReader();
		}
		graphReader.readEdgeList();
		graphReader.dump();
		
		M865Graph g = graphReader.getGraph();
		System.out.println(g);
		System.out.println("nC: " + g.getNumberOfComponents());
		System.out.println("Connected: " + g.isConnected());
		System.out.println("Has cycles: " + g.hasCycles());
		SortedSet vSet = g.getVertexSet();
		if (g.isConnected() && !g.hasCycles()) {
			System.out.println("This is a tree!");
			YITree tree = new YITree(g);
			System.out.println(tree);
			tree.buildRootedTree((M865Vertex)vSet.first());
			tree.printTree();
			System.out.println("Certificate: " + tree.getCertificate());
			tree.buildRootedTree(tree.getCenter());
			tree.printTree();
		} else {
			System.out.println("This is NOT a tree!");
		}			
		TreeMap tm = new TreeMap();
		Iterator iter = vSet.iterator();
		while (iter.hasNext()) {
			M865Vertex vertex = (M865Vertex)iter.next();
			System.out.println("Degree of "+vertex+" is " + g.degreeOf(vertex));
		}
	}
}