import java.io.*;
import java.util.*;

public class GracefulTreeSet {

	protected int nV;
	protected M865Vertex[] vertex;
	protected TreeSet vertexSet;
	protected TreeSet edgeList;
	protected TreeSet listOfCertificates;
	
	public GracefulTreeSet(int n) {
		nV = n;
		vertex = new M865Vertex[n];
		vertexSet = new TreeSet();
		for (int k = 0; k<n; k++) {
			M865Vertex v = new M865Vertex(new Integer(k));
			vertex[k] = v;
			vertexSet.add(v);
		}
		edgeList = new TreeSet();
		listOfCertificates = new TreeSet();
	}

	protected int getMax(int[] x) {
		int max = 0;
		for (int k=0; k<x.length; k++) {
			max = Math.max(max, x[k]);
		}
		return max;
	}
	
	protected int addEdge(int level, int s, int t, int[] oldForest, int count) {
		// s and t are the indices of the vertices to be added to the edge list
		M865Edge e = new M865Edge(vertex[s], vertex[t]);
		edgeList.add(e);
		if (level == 0) {
			count++;
			DWTree tree = new DWTree(vertexSet, edgeList);
			String certificate = tree.getCertificate();
			if (!listOfCertificates.contains(certificate)) {
				listOfCertificates.add(certificate);
				System.out.println(count + ": " + listOfCertificates.size() + ": " + certificate);
				// tree.buildRootedTree(tree.getCenter());
				// tree.printTree();
			}
		} else {
			int[] forest = new int[nV];
			for (int k=0; k<nV; k++) {
				forest[k] = oldForest[k];
			}
			int max = getMax(forest);
			for (int k=0; (k+level)<nV; k++) {
				s = k;
				t = k+level;
				int cs = forest[s];
				int ct = forest[t];
				// Check the four cases for this candidate edge
				if  (cs==0 && ct==0) {
					// New tree in the forest
					forest[s] = max+1;
					forest[t] = max+1;
					count = addEdge(level-1, s, t, forest, count);
					forest[s] = 0;
					forest[t] = 0;
				} else if (cs==0) {
					// Growing the tree from vertex t
					forest[s] = ct;
					count = addEdge(level-1, s, t, forest, count);
					forest[s] = 0;
				} else if (ct==0) {
					// Growing the tree from vertex s
					forest[t] = cs;
					count = addEdge(level-1, s, t, forest, count);
					forest[t] = 0;
				} else if (cs != ct) {
					// Merge two trees
					int lo = Math.min(cs, ct);
					int hi = Math.max(cs, ct);
					for (int j=0; j<nV; j++) {
						if (forest[j] == hi) {
							forest[j] = lo;
						}
					}
					count = addEdge(level-1, s, t, forest, count);
					for (int j=0; j<nV; j++) {
						forest[j] = oldForest[j];
					}
				} else {
					// Do nothing since this edge creates a cycle
				}
			}
		}
		edgeList.remove(e);
		return count;
	}
	
	public void buildTrees() {
		int[] forest = new int[nV];
		for (int k=0; k<nV; k++) {
			forest[k] = 0;
		}
		if (nV <  3) {
			System.out.println("Trivial tree with " + nV + " nodes.");
		} else {
			forest[0] = 1;
			forest[nV-1] = 1;
			M865Edge e = new M865Edge(vertex[0], vertex[nV-1]);
			edgeList.add(e);
			forest[nV-2] = 1;
			int count = addEdge(nV-3, 0, nV-2, forest, 0);
			System.out.println("Examined " + count + " gracefully labeled trees.");
			System.out.println("Found " + listOfCertificates.size() + " nonisomorphic trees.");
		}
	}
	
	public String toString() {
		String s ="Number of vertices: " + nV + "\n{ ";
		for (int k=0; k<nV; k++) {
			s += vertex[k].toString() + " ";
		}
		return s + "}";
	}
	
	public static void main(String[] args) {
		int nV = 5;
		if (0 < args.length) {
			nV = Integer.parseInt(args[0]);
		}
		GracefulTreeSet gts = new GracefulTreeSet(nV);
		System.out.println(gts);
		gts.buildTrees();
	}
}