import java.applet.Applet;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Image;

//import java.awt.Frame;
//import java.awt.event.WindowAdapter;
//import java.awt.event.WindowEvent;

import m865.shapepack.Rectangle;

/**
 * Applet/Application which visually solves the Hanoi puzzle.
 * @author Daniel Warner
 * @version 1.0  October 1, 2005
 */
public class HanoiApplet extends Applet implements Runnable {

	private static final long serialVersionUID = 1L;
	
	/**
	 * The maximum number of rings
	 */
	protected static final int MAXN = 7;
	/**
	 * default Width for the largest ring
	 */
    protected static final int defaultWidth = 180;
	/**
	 * default Height for all the rings
	 */
    protected static final int defaultHeight = 24;
    protected static final int clearance = 60 + defaultHeight;
    /**
     * The selector for the number of rings
     */
    protected Choice  numberChoice ;
    /**
     * The button for running the puzzle
     */
    protected Button  goButton;
    /**
     * The posts for stacking and displaying the rings
     */
    protected Post postA, postB, postC;
    protected Rectangle[] theRings = new Rectangle[MAXN];
    protected Rectangle movingRing;

	protected Image offscreenImg;
	protected Graphics offscreenG;
	protected int offscreenWidth;
	protected int offscreenHeight;
	
	protected Thread runner;
	protected int nRings;
	
    public HanoiApplet() {
    	int postWidth = 12;
    	int xa = postWidth + defaultWidth/2;
    	int rh = MAXN * (defaultHeight+1);
    	postA = new Post(xa,clearance, postWidth, rh, Color.black);
    	int xb = xa + defaultWidth + postWidth;
    	postB = new Post(xb,clearance, postWidth, rh, Color.black);
    	int xc = xb + defaultWidth + postWidth;
    	postC = new Post(xc,clearance, postWidth, rh, Color.black);
    	
    	int rw = defaultWidth;
    	theRings[0] = new Rectangle(0, 0, rw, defaultHeight, Color.orange);
    	rw -= 24;
    	theRings[1] = new Rectangle(0, 0, rw, defaultHeight, Color.green);
    	rw -= 24;
    	theRings[2] = new Rectangle(0, 0, rw, defaultHeight, Color.blue);
    	rw -= 24;
    	theRings[3] = new Rectangle(0, 0, rw, defaultHeight, Color.red);
    	rw -= 24;
    	theRings[4] = new Rectangle(0, 0, rw, defaultHeight, Color.yellow);
    	rw -= 24;
    	theRings[5] = new Rectangle(0, 0, rw, defaultHeight, Color.cyan);
    	rw -= 24;
    	theRings[6] = new Rectangle(0, 0, rw, defaultHeight, Color.magenta);
    }
    /**
     * Initializes the Graphical User Interface.
     */
    public void init() {

        numberChoice = new Choice();
        for (int k=1; k<=MAXN; k++) {
        	Integer nr = new Integer(k);
        	numberChoice.addItem(nr.toString());
        }
        add(numberChoice);    
        
        goButton = new Button("Go");
        goButton.setBackground(Color.white);
        add(goButton);
        System.out.println("Halfway through init");
        
    	offscreenWidth = this.getWidth();
    	offscreenHeight = this.getHeight();
		offscreenImg = createImage(offscreenWidth, offscreenHeight);
		offscreenG = offscreenImg.getGraphics();
		drawBackdrop();
	}
    
    /**
     * Visually solves the puzzle when the goButton is clicked.
     */
    public boolean action(Event e, Object arg) {
        if (e.target == goButton) {
            String numberString = numberChoice.getSelectedItem();
            nRings = Integer.parseInt(numberString);
            postA.clear();
            postB.clear();
            postC.clear();
            for (int k=0; k<nRings; k++) {
            	postA.push(theRings[k]);
            }
            drawBackdrop();
    		repaint();
    		
    		runner = new Thread(this);
    		runner.start();
        }
        return true;
	}

	public void run() {
		Hanoi(nRings, postA, postB, postC);
	}
	
	public void Hanoi(int n, Post a, Post b, Post c) {
		if (n==1) {
			// System.out.println("Move " + A + " to " + B);
			moveRing(a, b);
		} else {
			Hanoi(n-1, a, c, b);
			// System.out.println("Move " + A + " to " + B);
			moveRing(a, b);
			Hanoi(n-1, c, b, a);
		}
	}
    
    protected void moveRing(Post a, Post b) {
    	movingRing = a.pop();
    	drawBackdrop();
    	int xpos = movingRing.getX();
    	int ypos = movingRing.getY();
    	//Move the ring up
		while (clearance - defaultHeight < ypos) {
			ypos -= 6;
			movingRing.moveTo(xpos,ypos);
			repaint();
			try { Thread.sleep(50); }
			catch (InterruptedException e) { }
		}
		// Move the ring across
		int xGap = b.getX() - a.getX();
		int xTarget = xpos + xGap;
		if (xpos < xTarget) {
			while (xpos < xTarget) {
				xpos += 12;
				movingRing.moveTo(xpos,ypos);
				repaint();
				try { Thread.sleep(50); }
				catch (InterruptedException e) { }
			}
		} else {
			while (xTarget < xpos) {
				xpos -= 12;
				movingRing.moveTo(xpos,ypos);
				repaint();
				try { Thread.sleep(50); }
				catch (InterruptedException e) { }
			}
		}
		movingRing.moveTo(xTarget, ypos);
		// Move the ring down
		int yTarget = b.getY() + b.getNextRing() - movingRing.getHeight();
		while (ypos < yTarget) {
			ypos += 6;
			movingRing.moveTo(xpos,ypos);
			repaint();
			try { Thread.sleep(50); }
			catch (InterruptedException e) { }
		}
		movingRing.moveTo(xTarget,yTarget);
		b.push(movingRing);
		drawBackdrop();
		movingRing = null;
		repaint();
    }

    public void paint(Graphics g) {
		// Display the static portion of the puzzle
		g.drawImage(offscreenImg,0,0,this);
		// Draw the moving ring
		if (movingRing != null) {
			movingRing.draw(g);
		}
    }
    
    protected void drawBackdrop() {
		offscreenG.setColor(Color.white);
		offscreenG.fillRect(0, 0, offscreenWidth, offscreenHeight);
		postA.draw(offscreenG);
		postB.draw(offscreenG);
		postC.draw(offscreenG);
    }
    
	/**
	 * @param n - the number of rings
	 * @param a - the post where all the rings start out
	 * @param b - the post where all the rings should end up
	 * @param c - the post for temporary storage of the rings
	 */
//	public static void main(String args[]) {
//		if (args.length < 4) {
//			System.out.println("Usage: Hanoi n a b c");
//			System.exit(0);
//		}
//		int n = Integer.parseInt(args[0]);
//		String A = args[1];
//		String B = args[2];
//		String C = args[3];
//		
//		Hanoi(n, A, B, C);	
//
//	    // This applet can also run as a stand alone application
//	    public static void main(String args[]) {
//	        Frame f = new Frame();
//	        // Applet a = new SimpleDraw2(new ShapeFactory());
//	        Applet a = new SimpleDraw3(new MyFactory());
//	        f.add(a, "Center");
//	        a.init();
//	        f.setSize(400,400);
//	        f.show();
//	        f.addWindowListener(
//	            new WindowAdapter() {
//	                public void WindowClosing(WindowEvent e) {
//	                    System.exit(0);
//	                }
//	            }
//	        );
//	    }
//
//	}
}

