package m865.shapepack;

import java.awt.Color;
import java.awt.Graphics;

/** The concrete Shape subclass that defines rectangles.
 * 
 * @author  Daniel D. Warner 
 * 
 * @version  1.0 09/01/05
 * 
*/
public class Rectangle extends Shape {
	/**
	 * The generic Rectangle constructor
	 * 
	 * @param x horizontal coordinate of the upper left corner of the rectangle.
	 * @param y vertical coordinate of the upper left corner of the rectangle.
	 * @param w width of the rectangle.
	 * @param h height of the rectangle.
	 * @param c fill color for the rectangle. 
	 */
	public Rectangle(int x, int y, int w, int h, Color c) {
		super(x, y, w, h, c);
	}
	/**
	 * Constructor for a rectangle at the specified location with the default width, 
	 * height, and color.
	 * 
	 * @param x horizontal coordinate of the upper left corner of the rectangle.
	 * @param y vertical coordinate of the upper left corner of the rectangle.
	 */
	public Rectangle(int x, int y) {
		super(x, y, defaultWidth, defaultHeight, defaultColor);
	}
	/**
	 * Draws the rectangle
	 */
	public void draw(Graphics g) {
		g.setColor(this.color);
		g.fillRect(this.x, this.y, this.width, this.height);  
	}
	/**
	 * Calculates the area.
	 * 
	 * @return  the area of the rectangle
	 */
	public float area() {
		return( width*height );
	}
	/**
	 * Calculates the perimeter.
	 * 
	 * @return  the perimeter of the rectangle
	 */
	public float perimeter() {
		return( 2*(width + height) );
	}
	/**
	 * Generates the string describing the rectangle.
	 */
	public String toString() {
		return( "Rectangle:  " + super.toString() );
	}
}

