package m865.shapepack;

import java.awt.Color;
import java.awt.Graphics;

/** 
 * The Shape class is an abstract class which defines the fields, methods
 * and behavior common to all shapes.
 * 
 * @author  Daniel D. Warner 
 * 
 * @version  1.0 09/01/05
 * 
*/
public abstract class Shape {
	/**
	 * Constant specifying the default width of a shape.
	 */
	static public final int defaultWidth = 20;
	/**
	 * Constant specifying the default height of a shape.
	 */
	static public final int defaultHeight = 10;
	/**
	 * Constant specifying the default fill color for a shape.
	 */
	static public final Color defaultColor = Color.red;
	
	/**
	 * Horizontal coordinate of the upper left corner of the bounding box.
	 */
	protected int x;
	/**
	 * Vertical coordinate of the upper left corner of the bounding box.
	 */
	protected int y;
	/**
	 * Width of the bounding box.
	 */
	protected int width;
	/**
	 * Height of the bounding box.
	 */
	protected int height;
	/**
	 * Fill color for the shape
	 */
	protected Color color;

	/**
	 * The generic constructor for shape objects.
	 *  
	 * @param x horizontal coordinate of the upper left corner of the bounding box.
	 * @param y vertical coordinate of the upper left corner of the bounding box.
	 * @param w width of the bounding box.
	 * @param h height of the bounding box.
	 * @param c fill color for the shape. 
	 */
	public Shape(int x, int y, int w, int h, Color c) {
		this.x = x;
		this.y = y;
		this.width = w;
		this.height = h;
		this.color = c;
	}
	/**
	 * Moves the shape by setting the upper left corner of the bounding box to
	 * the specified coordinates.
	 * 
	 * @param x  the new horizontal coordinate of the upper left corner of the bounding box.
	 * @param y  the new vertical coordinate of the upper left corner of the bounding box.
	 */
	public void moveTo(int x, int y) {
		this.x = x;
		this.y = y;
	}
	/**
	 * 
	 * @return the horizontal coordinate of the upper left corner of the bounding box.
	 */
	public int getX() {
		return( x );
	}
	/**
	 * 
	 * @return the vertical coordinate of the upper left corner of the bounding box.
	 */
	public int getY() {
		return y;
	}
	/**
	 * 
	 * @return the width of the bounding box.
	 */
	public int getWidth() {
		return( width );
	}
	/**
	 * 
	 * @param w the new width of the bounding box.
	 */
	public void setWidth( int w ) {
		width = w;
	}
	/**
	 * 
	 * @return the height of the bounding box.
	 */
	public int getHeight() {
		return( height );
	}
	/**
	 * 
	 * @param h the new height of the bounding box.
	 */
	public void setHeight( int h ) {
		height = h;
	}
	/**
	 * 
	 * @return the fill color of the shape.
	 */
	public Color getColor() {
		return color;
	}
	/**
	 * 
	 * @param color the new fill color for the shape.
	 */
	public void setColor(Color color) {
		this.color = color;
	}
	/**
	 * Draws the shape.
	 * 
	 * @param g the graphics object on which the shape is to be drawn.
	 */
	abstract public void draw(Graphics g);
	/**
	 * Calculates the area of the shape.
	 * 
	 * @return the area
	 */
	abstract public float area();
	/**
	 * Calculates the perimeter of the shape.
	 * 
	 * @return the perimeter
	 */
	abstract public float perimeter();
	/**
	 * Generates a string describing the parameters of the shape.
	 */
	public String toString() {
		String myInfo;
		myInfo = "at (" + x + "," + y + "), ";
		myInfo += width + " pixels wide, " + height + " pixels high.\n";
		myInfo += color;
		return( myInfo );
	}
}
