import java.util.*;
 
public class DWvStack extends M865Stack {
	protected Vector v;
	
	public DWvStack(int capacity, int increment) {
		v = new Vector(capacity, increment);
	}
	
	public DWvStack(int capacity) {
		v = new Vector(capacity);
	}
	
	public DWvStack() {
		v = new Vector();
	}
	
	protected DWvStack(long hash, Vector v) {
		this.hash = hash;
		this.v = (Vector)v.clone();
	}
	
	public void push(Object obj) {
	// pushes an object onto the stack.
		hash = (hash + obj.hashCode()) % Integer.MAX_VALUE;
		v.add( obj );
	}
	
	public Object pop() {
	// removes and returns the object on the top of the stack.
	// returns null if the stack is empty.
		if (v.isEmpty()) {
			return null;
		} else {
			Object obj = v.remove(v.size()-1);
			hash = (hash - obj.hashCode()) % Integer.MAX_VALUE;
			return obj;
		}
	}
	
	public Object peek() {
	// returns the object on the top of the stack.
	// returns null if the stack is empty.
		if (v.isEmpty()) {
			return null;
		} else {
			return (v.lastElement());
		}
	}
	
	public void clear() {
	// Removes all of the elements from this collection (optional operation). 
		hash = 0;
		v.clear();
	}
		
	public Object clone() {
	// Returns a copy of this stack
		return (new DWvStack(hash, v));
	}
	
	public class DWvStackIterator implements Iterator {
		private DWvStack myS;
		private int index;
		private long check;
		
		DWvStackIterator(DWvStack s) {
			myS = s;
			index = s.v.size();
			check = s.hash;
		}

		public boolean hasNext() {
			//  Check that the stack has not been altered during the processing
			//  of this iterator
			if (check == myS.hash) {
				return (0 < index);
			} else {
				throw( new ConcurrentModificationException("This DWvStack has been altered"));
			}
		}
		
		public Object next() {
			if (check == myS.hash) {
				return myS.v.get(--index);
			} else {
				throw( new ConcurrentModificationException("This DWvQueue has been altered"));
			}
		}
		
		public void remove() {
	// Removes the last element returned by the iterator.
	// This is an optional operation which is not implemented for DWvStackIterator.
			throw( new UnsupportedOperationException("remove is not implemented for the general M865Stack"));
		}
	}

	public Iterator iterator() {
	// Returns an iterator over the elements in this collection.  
		return new DWvStackIterator(this);
	}
	
	public int size() {
	// Returns the number of elements in this collection.  
		return v.size();
	}

	public String toString() {
		String s = super.toString();
		return( "DWv" + s.substring(4, s.length()) );
	}
	
	public static void main(String[] args) {
		DWvStack s1 = new DWvStack();
		String name;
		
		s1.push("Alice");
		s1.add("Bob");
		s1.push("Carol");
		System.out.println(s1);
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("Empty? " + s1.isEmpty());
		 
		System.out.println("\ns1 Iterator test:");
		Iterator iter = s1.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		
		System.out.println("\nLooking at three stacks");
		DWvStack s2 = new DWvStack();
		s2.addAll(s1);
		System.out.println("\ns2 Iterator test:");
		iter = s2.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		DWvStack s3 = new DWvStack();
		s3.addAll(s2);
		System.out.println("\ns3 Iterator test:");
		iter = s3.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}

		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1 equals s3? " + s1.equals(s3));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());
		System.out.println("s3.hashCode: " + s3.hashCode());
		System.out.println("Is Bob in s1? " + s1.contains("Bob"));
		s1.add("Doug");
		System.out.println(s1.peek());
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());
		System.out.println("s3.hashCode: " + s3.hashCode());
		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1 equals s3? " + s1.equals(s3));
		System.out.println("Is s1 contained in s2? " + s2.containsAll(s1));
		System.out.println("Is s2 contained in s1? " + s1.containsAll(s2));
		System.out.println("Is s2 contained in s3? " + s3.containsAll(s3));
		
		System.out.println("\nLooking at cloning");
		DWvStack s4 = (DWvStack)s1.clone();
		System.out.println("s1 equals s4? " + s1.equals(s4));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s4.hashCode: " + s4.hashCode());
		System.out.println("s4.pop: " + s4.pop());
		System.out.println("s1 equals s4? " + s1.equals(s4));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s4.hashCode: " + s4.hashCode());

		System.out.println("\nLooking at an array and a stack");
		Object[] names = s1.toArray();
		for (int i=0; i<names.length; i++) {
			System.out.println(names[i]);
		}
		String[] moreNames = (String[])s1.toArray(new String[5]);
		System.out.println("moreNames.length: " + moreNames.length);
		for (int i=0; i<moreNames.length; i++) {
			System.out.println(moreNames[i]);
		}


		System.out.println("\nEmptying the stacks");
		System.out.println(s1.pop());
		System.out.println(s1.hashCode());
		System.out.println(s1.pop());
		System.out.println(s1.pop());
		System.out.println(s1.pop());
		System.out.println("Empty? " + s1.isEmpty());
		s2.clear();
		System.out.println("Empty? " + s2.isEmpty());
		System.out.println("s1 equals s2? " + s1.equals(s2));
		System.out.println("s1.hashCode: " + s1.hashCode());
		System.out.println("s2.hashCode: " + s2.hashCode());

		System.out.println("\nTesting the iterator for consistency checking");
		iter = s4.iterator();
		try {
			System.out.println(iter.next());
			System.out.println(iter.next());
			System.out.println("Pop: " + s4.pop());
			System.out.println(iter.next());
		} catch (ConcurrentModificationException e) {
			System.out.println("Concurrent Modification Exception: " + e.getMessage());
		}			
	}
}