/*
    Keyword Search Application
    
    The objective is to build a program which reads from the
    standard input. The first line will be a comma-separated
    list of keywords. The program is to read the remaining
    lines of input, identify every line that contains one
    or more of the keywords, and indicate which keywords are
    in that line.

    More specifically.
    1. Read the first line and build a queue containing the
       keywords.
    2. Build a Finite State Machine which will recognize the
       keywords.
       a. Determine all forward transitions and their states.
          Do this in a breadth-first fashion, so that the
          succeeeding step is simplified.
       b. Build the failure function and complete the finite
          state transition function.
       c. Build an appropriate action function.
    3. Read the remainder of the file character by character
       and process each character with your Finite State Machine.
    4. For each line, if the line contains one or more keywords,
       print the line number followed by a colon and a space, and
       then print the list of keywords in that line.

    Treat upper case and lower case characters as the same character.
    Assume that there will be no more than 40 keywords and that
    keywords will only contain the letters a thru z.

    I will send you one or more test files to run thru your program.
    You should turn in (1) a short discussion of your design and
    implementation, (2) a printout of your program, and (3) a
    printout of the test results.

*/

import java.io.*;
import java.util.*;

public class KeywordSearch {


    protected static int parent[];
    protected static int fail[];
    protected static int f[][];
    protected static String g[];

// parent.addElement(p);
// fail.addElement((f.elementAt(fail.elementAt(p)))[ch]);

    public static void main(String args[]) {
        InputStreamReader stdin = new InputStreamReader(System.in);
        int line=1, val=0, charCount=0;
        char c;
        Queue q = new Queue(25);
        String ss[];
        String w = "";
	    while (val != -1) {
            try {
                val = stdin.read();
                if (val != -1) {
                    c = (char)val;
                	if (c == ',') {
                	    q.enqueue(new StateStringPair(0,w));
                	    w = "";
                	} else if (c == '\n') {
                	    q.enqueue(new StateStringPair(0,w));
                	    w = "";
                	    line++;
                	    break;
                	} else {
                        w += c;
                        charCount++;
                    }
                }
            } catch (IOException e) {} 
	    }
        // System.out.println(q);
        
        //  charCount is an upper bound on the number of states.
        parent = new int[charCount];
        fail   = new int[charCount];
        f      = new int[charCount][27];
        g      = new String[charCount];
        ss     = new String[charCount];
        for (int j=0; j<charCount; j++) {
            parent[j] = 0;
            fail[j] = 0;
            for (int k=0; k<27; k++) {
                f[j][k] = 0;
            }
            g[j]  = "";
            ss[j] = "";
        }
		
        // Build the parent function and the forward transitions
        int maxState = 1, index, s;
        StateStringPair pair;
        while (!q.isEmpty()) {
            pair = (StateStringPair)q.dequeue();
            s = pair.getState();
            w = pair.getString();
            c = w.charAt(0);
            index = charEncode(c);
            if (0 == f[s][index]) {
                parent[maxState] = s;
                f[s][index] = maxState;
                ss[maxState] = ss[s] + c;
                s = maxState++;
            } else {
                s = f[s][index];
            }
            if (1 < w.length()) {
                q.enqueue(new StateStringPair(s,w.substring(1)));
            } else {
                g[s] = ss[s];
            }
            // System.out.println(q);
        }
        // Build the fail function and complete the state
        // transition function.
        int fs;
        for (s=0; s<maxState; s++) {
            if (parent[s]==0) {
                fs = fail[s] = 0;
            } else {
                int n = ss[s].length();
                index = charEncode(ss[s].charAt(n-1));
                fs = fail[s] = f[fail[parent[s]]][index];
            }
            if (0 < g[fs].length()) {
                if (0 < g[s].length()) {
                    g[s] += ", " + g[fs];;
                } else {
                    g[s] = g[fs];
                }
            }
            for (int k=0; k<27; k++) {
                if (f[s][k] == 0) {
                    f[s][k] = f[fs][k];
                }
            }
        }
        // Verification printout.
        for (s=0; s<maxState; s++) {
            if (9 < s) {
                System.out.println("fail(" + s + ") = " + fail[s]);
            } else {
                System.out.println("fail( " + s + ") = " + fail[s]);
            }
        }
        for (s=0; s<maxState; s++) {
            for (int k=0; k<27; k++) {
                if (9 < f[s][k]) {
                    System.out.print(" " + f[s][k]);
                } else {
                    System.out.print("  " + f[s][k]);
                }
            }
            System.out.println(" ");
        }
        for (s=0; s<maxState; s++) {
            if (9 < s) {
                System.out.println(" " + s + ": " + g[s]);
            } else {
                System.out.println("  " + s + ": " + g[s]);
            }
        }
        // The Finite State Machine has been built, now run it.
        boolean lineStarted = false;
        Integer lineNumber = new Integer(line);
        while (val != -1) {
            try {
                val = stdin.read();
                if (val == -1) {
                    System.out.println(" ");
                } else {
                    c = (char)val;
                    s = f[s][charEncode(c)];
                    if (0 < g[s].length()) {
                        if ( lineStarted ) {
                            System.out.print(", " + g[s]);
                        } else {
                            System.out.print("\n" + lineNumber + ": " + g[s]);
                            lineStarted = true;
                        }
                    }
                    if (c == '\n') {
                        lineStarted = false;
                        lineNumber = new Integer(++line);
                    }
                }
            } catch (IOException e) {} 
        }
    }        
    
    
    public static int charEncode(char c) {
        char ch = Character.toLowerCase(c);
        if (('a'<=ch) && (ch<='z')) {
            return (Character.getNumericValue(ch) - Character.getNumericValue('a'));
        } else {
            return 26;
        }
    }
 
}
        
