import java.io.*;
import java.util.*;

public class KeywordTokenizer {
    public static final int KW_EOF=-1, KW_TOK=1;
    public int lineno = 1;
    public String keyword;
    
    protected InputStreamReader stdin;
    protected int f[][];
    protected String g[];
    protected int state;
    
    public KeywordTokenizer(InputStream inComing) {
        stdin = new InputStreamReader(inComing);
        this.init();
    }
    
    public void init() {
        int val=0, charCount=0, parent[], fail[];
        char c;
        Queue q = new Queue(25);
        String ss[];
        String w = "";

	    while (val != -1) {
            try {
                val = stdin.read();
                if (val != -1) {
                    c = (char)val;
                	if (c == ',') {
                	    q.enqueue(new StateStringPair(0,w));
                	    w = "";
                	} else if (c == '\n') {
                	    q.enqueue(new StateStringPair(0,w));
                	    w = "";
                	    lineno++;
                	    break;
                	} else {
                        w += c;
                        charCount++;
                    }
                }
            } catch (IOException e) {} 
	    }
        // System.out.println(q);
        
        //  charCount is an upper bound on the number of states.
        parent = new int[charCount];
        fail   = new int[charCount];
        f      = new int[charCount][27];
        g      = new String[charCount];
        ss     = new String[charCount];
        for (int j=0; j<charCount; j++) {
            parent[j] = 0;
            fail[j] = 0;
            for (int k=0; k<27; k++) {
                f[j][k] = 0;
            }
            g[j]  = "";
            ss[j] = "";
        }
		
        // Build the parent function and the forward transitions
        int maxState = 1, index, s;
        StateStringPair pair;
        while (!q.isEmpty()) {
            pair = (StateStringPair)q.dequeue();
            s = pair.getState();
            w = pair.getString();
            c = w.charAt(0);
            index = charEncode(c);
            if (0 == f[s][index]) {
                parent[maxState] = s;
                f[s][index] = maxState;
                ss[maxState] = ss[s] + c;
                s = maxState++;
            } else {
                s = f[s][index];
            }
            if (1 < w.length()) {
                q.enqueue(new StateStringPair(s,w.substring(1)));
            } else {
                g[s] = ss[s];
            }
            // System.out.println(q);
        }
        // Build the fail function and complete the state
        // transition function.
        int fs;
        for (s=0; s<maxState; s++) {
            if (parent[s]==0) {
                fs = fail[s] = 0;
            } else {
                int n = ss[s].length();
                index = charEncode(ss[s].charAt(n-1));
                fs = fail[s] = f[fail[parent[s]]][index];
            }
            if (0 < g[fs].length()) {
                if (0 < g[s].length()) {
                    g[s] += ", " + g[fs];;
                } else {
                    g[s] = g[fs];
                }
            }
            for (int k=0; k<27; k++) {
                if (f[s][k] == 0) {
                    f[s][k] = f[fs][k];
                }
            }
        }
        // Verification printout.
        for (s=0; s<maxState; s++) {
            if (9 < s) {
                System.out.println("fail(" + s + ") = " + fail[s]);
            } else {
                System.out.println("fail( " + s + ") = " + fail[s]);
            }
        }
        for (s=0; s<maxState; s++) {
            for (int k=0; k<27; k++) {
                if (9 < f[s][k]) {
                    System.out.print(" " + f[s][k]);
                } else {
                    System.out.print("  " + f[s][k]);
                }
            }
            System.out.println(" ");
        }
        for (s=0; s<maxState; s++) {
            if (9 < s) {
                System.out.println(" " + s + ": " + g[s]);
            } else {
                System.out.println("  " + s + ": " + g[s]);
            }
        }
        state = 0;
    }
    
    public int nextKeyword() throws IOException {
    // The Finite State Machine has been built, now run it.
        int val;
        char c;
        while (state != KW_EOF) {
            val = stdin.read();
            if (val == -1) {
                state = KW_EOF;
                return KW_EOF;
            } else {
                c = (char)val;
                if (c == '\n') {
                    lineno++;
                }
                state = f[state][charEncode(c)];
                if (0 < g[state].length()) {
                    keyword = g[state];
                    return KW_TOK;
                }
            }
        }
        return KW_EOF;
    }        
    
    
    public static int charEncode(char c) {
        char ch = Character.toLowerCase(c);
        if (('a'<=ch) && (ch<='z')) {
            return (Character.getNumericValue(ch) - Character.getNumericValue('a'));
        } else {
            return 26;
        }
    }
}

