/*        
    This program reads a file and prints out each
    numeric litteral.
    
    Daniel D. Warner   10/12/98
*/

import java.io.*;
import java.util.*;

public class XMLRecursor {
    
    public static void main(String args[]) {
        InputStreamReader stdin = new InputStreamReader(System.in);
        XMLTokenizer reader = new XMLTokenizer(stdin);
        XMLLine myLine = new XMLLine("");
        
        int token = 0;
        while (token != XMLTokenizer.XML_EOF) {
            try {
                token = reader.getNextToken();
                // System.out.print( Integer.toString(token)); // for debugging.
                if ( token == XMLTokenizer.XML_EOF) {
                    myLine.flush();
                    System.out.println("EOF");
                } else if (token == XMLTokenizer.XML_STUFF) {
                    myLine.add(reader.sval + " ");
                } else if (token == XMLTokenizer.XML_STARTTAG) {
                    myLine.flush();
                    parse1("", reader);
                } else if (token == XMLTokenizer.XML_SIMPLETAG) {
                    myLine.flush();
                    parse2("", reader);
                } else if (token == XMLTokenizer.XML_ENDTAG) {
                    myLine.output(reader.sval);
                    System.out.println("*** Syntax error - Too Many EndTags ***");
                } else if (token == XMLTokenizer.XML_BADTAG) {
                    myLine.output(reader.sval);
                    System.out.println("*** Syntax error - Bad Tag ***");
                } else if (token == XMLTokenizer.XML_BADATTRLIST) {
                    myLine.output(reader.sval);
                    System.out.println("*** Syntax error - Bad Attribute List ***");
                }
            } catch (IOException e) {} 
        }
    }
    
    static void parse1(String indent, XMLTokenizer reader) {
    	// This routine is called when the XMLTokenizer has a StartTag.

    	// Save the tag name, print the tag, and indent the left margin.
    	String myTag = reader.tagName;
        XMLLine myLine = new XMLLine( indent );
        myLine.output(reader.sval);
        myLine.bumpIndent();
        
        int token = 0;
        while (token != XMLTokenizer.XML_EOF) {
            try {
                token = reader.getNextToken();
                // System.out.print( Integer.toString(token)); // for debugging.
                if ( token == XMLTokenizer.XML_EOF) {
                    myLine.flush();
                    System.out.println("*** Syntax error - Too Many Start Tags ***");
                } else if (token == XMLTokenizer.XML_STUFF) {
                    myLine.add(reader.sval + " ");
                } else if (token == XMLTokenizer.XML_STARTTAG) {
                    myLine.flush();
                    parse1(myLine.getIndent(), reader);
                } else if (token == XMLTokenizer.XML_SIMPLETAG) {
                    myLine.flush();
                    parse2(myLine.getIndent(), reader);
                } else if (token == XMLTokenizer.XML_ENDTAG) {
                    if (!myTag.equalsIgnoreCase(reader.tagName)) {
                        myLine.output(reader.sval);
                        System.out.println("*** Syntax error - Mismatched Tags ***");
                    } else {
                        myLine.flush();
                    }
                    break;
                } else if (token == XMLTokenizer.XML_BADTAG) {
                    myLine.output(reader.sval);
                    System.out.println("*** Syntax error - Bad Tag ***");
                    break;
                } else if (token == XMLTokenizer.XML_BADATTRLIST) {
                    myLine.output(reader.sval);
                    System.out.println("*** Syntax error - Bad Attribute List ***");
                    break;
                }
            } catch (IOException e) {} 
        }
    }
 	
 	static void parse2(String indent, XMLTokenizer reader) {
    	// This routine is called when the XMLTokenizer has a SimpleTag.

    	// Print the tag and return.
        XMLLine myLine = new XMLLine( indent );
        myLine.output(reader.sval);
    }

}
