/*        
    This class implements a tokenizer suitable for parsing XML.
    
    Daniel D. Warner   10/17/98
*/

import java.io.*;

public class XMLTokenizer {

    public static final int XML_EOF=-1, XML_STUFF=-2, XML_STARTTAG=-3,
        XML_ENDTAG=-4, XML_SIMPLETAG=-5, XML_BADTAG=-6, XML_BADATTRLIST=-7;
    
    public String tagName, sval;
    
    protected StreamTokenizer reader;
    protected int state, token;
    protected String stmp;
    
    // Define the state transition function
    // There are 20 states adn the input alphabet is
    protected int f[][] =
    //      <, wrd, /,  >,  =, str, other.
        {{  1,  0,  0,  0,  0,  0,  0},  // state  0 - Go forward on '<'
         { 17,  2, 12, 15, 15, 15, 15},  // state  1 - <  forward on wrd or '/'
         { 17,  6,  4,  3, 15, 15, 15},  // state  2 - <wrd
         {  1,  0,  0,  0,  0,  0,  0},  // state  3 - <wrd>
         { 17, 15, 15,  5, 15, 15, 15},  // state  4 - <wrd/
         {  1,  0,  0,  0,  0,  0,  0},  // state  5 - <wrd/>
         { 18, 16, 16, 16,  7, 16, 16},  // state  6 - <wrd (wrd=str)* wrd
         { 18, 16, 16, 16, 16,  8, 16},  // state  7 - <wrd (wrd=str)* wrd=
         { 17,  6, 10,  9, 15, 15, 15},  // state  8 - <wrd (wrd=str)* wrd=str
         {  1,  0,  0,  0,  0,  0,  0},  // state  9 - <wrd (wrd=str)* wrd=str>
         { 17, 15, 15, 11, 15, 15, 15},  // state 10 - <wrd (wrd=str)* wrd=str/
         {  1,  0,  0,  0,  0,  0,  0},  // state 11 - <wrd (wrd=str)* wrd=str/>
         { 17, 13, 15, 15, 15, 15, 15},  // state 12 - </
         { 17, 15, 15, 14, 15, 15, 15},  // state 13 - </wrd
         {  1,  0,  0,  0,  0,  0,  0},  // state 14 - <wrd/>
         {  1,  0,  0,  0,  0,  0,  0},  // state 15 - Bad Tag - Mirrors 0
         {  1,  0,  0,  0,  0,  0,  0},  // state 16 - Bad Attr List - Mirrors 0
         { 17,  2, 12, 15, 15, 15, 15},  // state 17 - Bad Tag - Mirrors 1
         { 17,  2, 12, 15, 15, 15, 15},  // state 18 - Bad Attr List - Mirrors 1
         { 19, 19, 19, 19, 19, 19, 19}}; // state 19 - EOF


    public XMLTokenizer(InputStreamReader streamReader) {
        reader = new StreamTokenizer(streamReader);
        reader.eolIsSignificant(false);  // The end of line is just white space.
        reader.ordinaryChar('/');        // Not to be used as a comment character.
        reader.ordinaryChar('\'');       // not to be used for a string or comment.
        
        state = 0;
        stmp = "";

    }
    
    public int getNextToken() throws IOException {
        int ch;
        boolean stillLooking = true;
        
        sval = "";
        tagName = "";
        while( stillLooking ) {
            ch = 6;
            if (state == 19) {
                return XML_EOF;
            }
            token = reader.nextToken();
            if (token == StreamTokenizer.TT_EOF) {
                state = 19;
            } else if (token == (int)'<') {
                stmp = "<";
                ch = 0;
            } else if (token == StreamTokenizer.TT_WORD) {
                stmp = reader.sval;
                ch = 1;
            } else if (token == (int)'/') {
                stmp = "/";
                ch = 2;
            } else if (token == (int)'>') {
                stmp = ">";
                ch = 3;
            } else if (token == (int)'=') {
                stmp = "=";
                ch = 4;
            } else if (token == (int)'"') {
                stmp = reader.sval;
                ch = 5;
            } else if (token == StreamTokenizer.TT_NUMBER) {
                stmp = Double.toString(reader.nval);
            } else {
                stmp = String.valueOf((char)token);
            }
            // System.out.println("old state: " + Integer.toString(state) + "    ch: " + Integer.toString(ch));            
            state = f[state][ch];
            // System.out.println("new state: " + Integer.toString(state) + "   " + stmp);
            
            switch ( state ) {
                case 0: {
                    sval += stmp;
                    return XML_STUFF;
                }
                case 2: case 13: {
                    sval += stmp;
                    tagName = reader.sval;
                    break;
                }
                case 3: case 9: {
                    sval += stmp;
                    return XML_STARTTAG;
                }
                case 1: case 4: case 7: case 10: case 12: {
                    sval += stmp;
                    break;
                }
                case 5: case 11: {
                    sval += stmp;
                    return XML_SIMPLETAG;
                }
                case 6: {
                    sval += " " + stmp;
                    break;
                }
                case 8: {
                    sval += "\"" + stmp + "\"";
                    break;
                }
                case 14: {
                    sval += stmp;
                    return XML_ENDTAG;
                }
                case 15: case 17: {
                    sval += stmp;
                    return XML_BADTAG;
                }
                case 16: case 18: {
                    sval += stmp;
                    return XML_BADATTRLIST;
                }
            }
        }
        // These statements should never get executed.
        sval += stmp;
        return XML_STUFF;
    }
}

                
                
                
    
